@@ -46,8 +46,8 @@ config.repository = "."
 config.skip_dirs = (
 )
 config.skip_file_ext = (".pyc",".gif", ".png")
-config.simulation = False
-#config.simulation = True
+#config.simulation = False
+config.simulation = True
 
 
 if __name__ == "__main__":
